@@ -570,7 +570,7 @@ def export_graph(graph_in, base_dir=None, show=False, use_execgraph=False,
     logger.info('Creating detailed dot file: %s' % outfname)
     _write_detailed_dot(graph, outfname)
     cmd = 'dot -T%s -O %s' % (format, outfname)
-    res = CommandLine(cmd).run()
+    res = CommandLine(cmd, terminal_output='allatonce').run()
     if res.runtime.returncode:
         logger.warn('dot2png: %s', res.runtime.stderr)
     pklgraph = _create_dot_graph(graph, show_connectinfo, simple_form)
@@ -581,7 +581,7 @@ def export_graph(graph_in, base_dir=None, show=False, use_execgraph=False,
     nx.write_dot(pklgraph, outfname)
     logger.info('Creating dot file: %s' % outfname)
     cmd = 'dot -T%s -O %s' % (format, outfname)
-    res = CommandLine(cmd).run()
+    res = CommandLine(cmd, terminal_output='allatonce').run()
     if res.runtime.returncode:
         logger.warn('dot2png: %s', res.runtime.stderr)
     if show:
@@ -634,7 +634,7 @@ def walk_outputs(object):
             if isdefined(val):
                 out.extend(walk_outputs(val))
     else:
-        if isdefined(object) and isinstance(object, str):
+        if isdefined(object) and isinstance(object, basestring):
             if os.path.islink(object) or os.path.isfile(object):
                 out = [(filename, 'f') for filename in get_all_files(object)]
             elif os.path.isdir(object):
@@ -682,12 +682,22 @@ def clean_working_directory(outputs, cwd, inputs, needed_outputs, config,
     logger.debug('Needed files: %s' % (';'.join(needed_files)))
     logger.debug('Needed dirs: %s' % (';'.join(needed_dirs)))
     files2remove = []
-    for f in walk_files(cwd):
-        if f not in needed_files:
-            if len(needed_dirs) == 0:
-                files2remove.append(f)
-            elif not any([f.startswith(dirname) for dirname in needed_dirs]):
-                files2remove.append(f)
+    if str2bool(config['execution']['remove_unnecessary_outputs']):
+        for f in walk_files(cwd):
+            if f not in needed_files:
+                if len(needed_dirs) == 0:
+                    files2remove.append(f)
+                elif not any([f.startswith(dirname) for dirname in needed_dirs]):
+                    files2remove.append(f)
+    else:
+        if not str2bool(config['execution']['keep_inputs']):
+            input_files = []
+            inputdict = inputs.get()
+            input_files.extend(walk_outputs(inputdict))
+            input_files = [path for path, type in input_files if type == 'f']
+            for f in walk_files(cwd):
+                if f in input_files and f not in needed_files:
+                    files2remove.append(f)
     logger.debug('Removing files: %s' % (';'.join(files2remove)))
     for f in files2remove:
         os.remove(f)
@@ -719,6 +729,8 @@ def merge_dict(d1, d2, merge=lambda x, y: y):
     if not isinstance(d1, dict):
         return merge(d1, d2)
     result = dict(d1)
+    if d2 is None:
+        return result
     for k, v in d2.iteritems():
         if k in result:
             result[k] = merge_dict(result[k], v, merge=merge)
